@@ -723,8 +723,7 @@ static int gfft2_move_to (FT_Vector * to, void * user)
 
 	p.x = to->x * od->t[0] + to->y * od->t[2];
 	p.y = to->x * od->t[1] + to->y * od->t[3];
-
-	if ((p.x != s->x3) || (p.y != s->y3)) {
+	if (od->end == 0 || (p.x != s->x3) || (p.y != s->y3)) {
 		od->bp[od->end].code = ART_MOVETO;
 		od->bp[od->end].x3 = to->x * od->t[0] + to->y * od->t[2];
 		od->bp[od->end].y3 = to->x * od->t[1] + to->y * od->t[3];
@@ -1189,6 +1188,7 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso, GPFontEntryT1 *t1)
 			}
 		}
 	} else {
+		gf_pso_ensure_space (pso, st.st_size);
 		memcpy (pso->buf, fbuf, st.st_size);
 		pso->buf[st.st_size] = '\0';
 		pso->length = st.st_size;
@@ -1208,9 +1208,9 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso, GPFontEntryT1 *t1)
 			gchar c[256];
 			FT_Error status;
 			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
-			status = FT_Get_Glyph_Name (pso->face->ft_face, glyph, c, 256);
+			status = FT_Get_Glyph_Name (pso->face->ft_face, g, c, 256);
 			if (status != FT_Err_Ok) {
-				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, t1->pfb.name);
+				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, g, t1->pfb.name);
 #if 0
 				gff_pso_ensure_buffer_empty (pso);
 				return;
@@ -1500,7 +1500,7 @@ gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...)
 	gchar *text;
 	gint len;
 	
-	oldlocale = setlocale (LC_NUMERIC, NULL);
+	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 		
 	va_start (arguments, format);
@@ -1514,6 +1514,7 @@ gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...)
 	g_free (text);
 
 	setlocale (LC_NUMERIC, oldlocale);
+	g_free (oldlocale);
 }
 
 static void
