@@ -32,7 +32,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <sys/time.h>
+#include <time.h>
 #include <errno.h>
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_vpath.h>
@@ -121,6 +121,8 @@ static void
 gnome_print_context_init (GnomePrintContext *pc)
 {
 	pc->gc = gp_gc_new ();
+	pc->level = 0;
+	pc->has_page = FALSE;
 
 	pc->output = GNOME_PRINT_OUTPUT_NULL;
 	pc->command = NULL;
@@ -148,7 +150,9 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 	
 	g_return_val_if_fail (printer != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINTER (printer), NULL);
-	g_return_val_if_fail (paper_size != NULL, NULL);
+
+	/* fixme: Not too beautiful, but there were people with missing paper.conf */
+	if (!paper_size) paper_size = "A4";
 
 #ifdef ENABLE_LIBGPA
 	/* If print to file pop up a dialog */
@@ -274,13 +278,25 @@ gnome_print_context_finalize (GtkObject *object)
 
 #define PRINT_CLASS(pc) GNOME_PRINT_CONTEXT_CLASS(GTK_OBJECT(pc)->klass)
 
+/* Helper to overcome broken application code */
+static void
+gnome_print_check_page (GnomePrintContext *ctx)
+{
+	if (!ctx->has_page) {
+		g_warning ("Application is sending data but did not call 'beginpage'");
+		gnome_print_beginpage (ctx, "Unnamed page");
+	}
+}
+
 int
 gnome_print_newpath (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->newpath)
 		ret =  PRINT_CLASS(pc)->newpath (pc);
@@ -295,8 +311,10 @@ gnome_print_moveto (GnomePrintContext *pc, double x, double y)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->moveto)
 		ret = PRINT_CLASS(pc)->moveto (pc, x, y);
@@ -311,9 +329,11 @@ gnome_print_lineto (GnomePrintContext *pc, double x, double y)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->lineto)
 		ret = PRINT_CLASS(pc)->lineto (pc, x, y);
@@ -328,9 +348,11 @@ gnome_print_curveto (GnomePrintContext *pc, double x1, double y1, double x2, dou
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->curveto)
 		ret = PRINT_CLASS(pc)->curveto (pc, x1, y1, x2, y2, x3, y3);
@@ -345,9 +367,12 @@ gnome_print_closepath (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->closepath)
 		ret = PRINT_CLASS(pc)->closepath (pc);
@@ -362,8 +387,10 @@ gnome_print_setrgbcolor (GnomePrintContext *pc, double r, double g, double b)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setrgbcolor)
 		ret = PRINT_CLASS(pc)->setrgbcolor (pc, r, g, b);
@@ -378,9 +405,12 @@ gnome_print_fill (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	gp_gc_close_all (pc->gc);
 
@@ -397,9 +427,12 @@ gnome_print_eofill (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	gp_gc_close_all (pc->gc);
 
@@ -416,9 +449,11 @@ gnome_print_setlinewidth (GnomePrintContext *pc, double width)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (width >= 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setlinewidth)
 		ret = PRINT_CLASS(pc)->setlinewidth (pc, width);
@@ -433,11 +468,11 @@ gnome_print_setmiterlimit (GnomePrintContext *pc, double limit)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (limit >= 1.0, GNOME_PRINT_ERROR_BADVALUE);
-
-	gp_gc_set_miterlimit (pc->gc, limit);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setmiterlimit)
 		ret = PRINT_CLASS(pc)->setmiterlimit (pc, limit);
@@ -452,9 +487,11 @@ gnome_print_setlinejoin (GnomePrintContext *pc, int jointype)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail ((jointype >= 0) && (jointype < 3), GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setlinejoin)
 		ret = PRINT_CLASS(pc)->setlinejoin (pc, jointype);
@@ -469,9 +506,11 @@ gnome_print_setlinecap (GnomePrintContext *pc, int captype)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail ((captype >= 0) && (captype < 3), GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	gp_gc_set_linecap (pc->gc, captype);
 
@@ -486,10 +525,11 @@ gnome_print_setdash (GnomePrintContext *pc, int n_values, const double *values,
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-
-	if (n_values != 0) g_return_val_if_fail (values != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (!n_values || (values != NULL), GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setdash)
 		ret = PRINT_CLASS(pc)->setdash (pc, n_values, values, offset);
@@ -504,9 +544,12 @@ gnome_print_strokepath (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->strokepath)
 		ret = PRINT_CLASS(pc)->strokepath (pc);
@@ -521,8 +564,12 @@ gnome_print_stroke (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->stroke)
 		ret = PRINT_CLASS(pc)->stroke (pc);
@@ -537,10 +584,12 @@ gnome_print_setfont (GnomePrintContext *pc, GnomeFont *font)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (font != NULL, GNOME_PRINT_ERROR_BADVALUE);
 	g_return_val_if_fail (GNOME_IS_FONT (font), GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->setfont)
 		ret = PRINT_CLASS(pc)->setfont (pc, font);
@@ -553,9 +602,12 @@ gnome_print_setfont (GnomePrintContext *pc, GnomeFont *font)
 int
 gnome_print_show (GnomePrintContext *pc, char const *text)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
 	g_return_val_if_fail (text != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	return gnome_print_show_sized (pc, text, strlen (text));
 }
@@ -566,9 +618,13 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 	const GnomeFont *font;
 	const char *invalid;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (text != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
+	g_return_val_if_fail (text != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (bytes >= 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (bytes < 1) return GNOME_PRINT_OK;
 
@@ -595,9 +651,13 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 int
 gnome_print_show_ucs4 (GnomePrintContext *pc, guint32 *buf, gint length)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (buf != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (gp_gc_has_currentpoint (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPOINT);
+	g_return_val_if_fail (buf != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (length >= 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (length < 1) return GNOME_PRINT_OK;
 
@@ -630,9 +690,11 @@ gnome_print_concat (GnomePrintContext *pc, const double matrix[6])
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (matrix != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (matrix != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->concat)
 		ret = PRINT_CLASS(pc)->concat (pc, matrix);
@@ -647,13 +709,16 @@ gnome_print_gsave (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->gsave)
 		ret = PRINT_CLASS(pc)->gsave (pc);
 
 	gp_gc_gsave (pc->gc);
+	pc->level += 1;
 
 	return ret;
 }
@@ -663,13 +728,18 @@ gnome_print_grestore (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->level > 0, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
+
 
 	if (PRINT_CLASS (pc)->grestore)
 		ret = PRINT_CLASS(pc)->grestore (pc);
 
 	gp_gc_grestore (pc->gc);
+	pc->level -= 1;
 
 	return ret;
 }
@@ -679,15 +749,20 @@ gnome_print_clip (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	gp_gc_close_all (pc->gc);
 
 	if (PRINT_CLASS (pc)->clip)
 		ret = PRINT_CLASS(pc)->clip (pc, ART_WIND_RULE_NONZERO);
 
+	gp_gc_newpath (pc->gc);
+
 	return ret;
 }
 
@@ -696,24 +771,33 @@ gnome_print_eoclip (GnomePrintContext *pc)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (gp_gc_has_currentpath (pc->gc), GNOME_PRINT_ERROR_NOCURRENTPATH);
+	g_return_val_if_fail (gp_gc_currentpath_points (pc->gc) > 1, GNOME_PRINT_ERROR_NOCURRENTPATH);
+	gnome_print_check_page (pc);
 
 	gp_gc_close_all (pc->gc);
 
 	if (PRINT_CLASS (pc)->clip)
 		ret = PRINT_CLASS(pc)->clip (pc, ART_WIND_RULE_ODDEVEN);
 
+	gp_gc_newpath (pc->gc);
+
 	return ret;
 }
 
 int
 gnome_print_grayimage (GnomePrintContext *pc, const char *data, int width, int height, int rowstride)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (data != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (width > 0, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (height > 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->grayimage)
 		return PRINT_CLASS(pc)->grayimage (pc, data, width, height, rowstride);
@@ -724,9 +808,13 @@ gnome_print_grayimage (GnomePrintContext *pc, const char *data, int width, int h
 int
 gnome_print_rgbimage (GnomePrintContext *pc, const char *data, int width, int height, int rowstride)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (data != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (width > 0, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (height > 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->rgbimage)
 		return PRINT_CLASS(pc)->rgbimage (pc, data, width, height, rowstride);
@@ -741,9 +829,13 @@ gnome_print_rgbaimage (GnomePrintContext *pc, const char *data, int width, int h
 	const guchar * s;
 	gint x, y, alpha, tmp, ret;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (data != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (data != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (width > 0, GNOME_PRINT_ERROR_BADVALUE);
+	g_return_val_if_fail (height > 0, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->rgbaimage != NULL) {
 
@@ -785,26 +877,34 @@ gnome_print_rgbaimage (GnomePrintContext *pc, const char *data, int width, int h
 int
 gnome_print_pixbuf (GnomePrintContext *pc, GdkPixbuf *pixbuf)
 {
-       if (gdk_pixbuf_get_has_alpha (pixbuf))
-               return gnome_print_rgbaimage  (pc,
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pixbuf != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
+
+	if (gdk_pixbuf_get_has_alpha (pixbuf))
+		return gnome_print_rgbaimage  (pc,
+					       gdk_pixbuf_get_pixels    (pixbuf),
+					       gdk_pixbuf_get_width     (pixbuf),
+					       gdk_pixbuf_get_height    (pixbuf),
+					       gdk_pixbuf_get_rowstride (pixbuf));
+	else
+		return gnome_print_rgbimage  (pc,
 					      gdk_pixbuf_get_pixels    (pixbuf),
 					      gdk_pixbuf_get_width     (pixbuf),
 					      gdk_pixbuf_get_height    (pixbuf),
 					      gdk_pixbuf_get_rowstride (pixbuf));
-       else
-               return gnome_print_rgbimage  (pc,
-					     gdk_pixbuf_get_pixels    (pixbuf),
-					     gdk_pixbuf_get_width     (pixbuf),
-					     gdk_pixbuf_get_height    (pixbuf),
-					     gdk_pixbuf_get_rowstride (pixbuf));
 }
 
 int
 gnome_print_textline (GnomePrintContext *pc, GnomeTextLine *line)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (line != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (line != NULL, GNOME_PRINT_ERROR_BADVALUE);
+	gnome_print_check_page (pc);
 
 	if (PRINT_CLASS (pc)->textline)
 		return PRINT_CLASS(pc)->textline (pc, line);
@@ -818,8 +918,19 @@ gnome_print_showpage (GnomePrintContext *pc)
 	const GnomeFont * font;
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
+
+	if (pc->level > 0) {
+		g_warning ("Application is trying to print page with nonempty gsave stack");
+		while (pc->level > 0) {
+			gint ret;
+			ret = gnome_print_grestore (pc);
+			g_return_val_if_fail (ret >= 0, ret);
+		}
+	}
 
 	if (PRINT_CLASS (pc)->showpage)
 		ret = PRINT_CLASS(pc)->showpage (pc);
@@ -831,14 +942,20 @@ gnome_print_showpage (GnomePrintContext *pc)
 	gp_gc_set_font (pc->gc, (GnomeFont *) font);
 	gnome_font_unref (font);
 
+	pc->has_page = FALSE;
+
 	return ret;
 }
 
 int
 gnome_print_beginpage (GnomePrintContext *pc, const char *name_of_this_page)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (!pc->has_page, GNOME_PRINT_ERROR_UNKNOWN);
+
+	pc->has_page = TRUE;
 
 	if (PRINT_CLASS (pc)->beginpage)
 		return PRINT_CLASS(pc)->beginpage (pc, name_of_this_page);
@@ -851,8 +968,12 @@ gnome_print_setopacity (GnomePrintContext *pc, double opacity)
 {
 	gint ret = GNOME_PRINT_OK;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
+
+	opacity = CLAMP (opacity, 0.0, 1.0);
 
 	if (PRINT_CLASS (pc)->setopacity)
 		ret = PRINT_CLASS(pc)->setopacity (pc, opacity);
@@ -867,8 +988,11 @@ gnome_print_context_close (GnomePrintContext *pc)
 {
 	gint ret;
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+
+	if (pc->has_page) g_warning ("Closing print context with open page");
+	if (pc->level > 0) g_warning ("Closing print context with nonempty stack");
 
 	if (PRINT_CLASS (pc)->close) {
 		ret = PRINT_CLASS(pc)->close (pc);
@@ -910,8 +1034,10 @@ gnome_print_scale (GnomePrintContext *pc, double sx, double sy)
 {
 	double dst[6];
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	art_affine_scale (dst, sx, sy);
 	return gnome_print_concat (pc, dst);
@@ -922,8 +1048,10 @@ gnome_print_rotate (GnomePrintContext *pc, double theta)
 {
 	double dst[6];
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	art_affine_rotate (dst, theta);
 	return gnome_print_concat (pc, dst);
@@ -934,8 +1062,10 @@ gnome_print_translate (GnomePrintContext *pc, double x, double y)
 {
 	double dst[6];
 
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	gnome_print_check_page (pc);
 
 	art_affine_translate (dst, x, y);
 	return gnome_print_concat (pc, dst);
@@ -950,9 +1080,10 @@ gnome_print_translate (GnomePrintContext *pc, double x, double y)
 int
 gnome_print_context_open_file (GnomePrintContext *pc, const char *filename)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
-	g_return_val_if_fail (filename != NULL, -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (filename != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	if (filename[0] == '|') {
 		/* We are pipe */
@@ -1008,9 +1139,9 @@ gnome_print_context_open_file (GnomePrintContext *pc, const char *filename)
 int
 gnome_print_context_write_file (GnomePrintContext *pc, const void *buf, size_t size)
 {
-	g_return_val_if_fail (pc != NULL, 0);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), 0);
-	g_return_val_if_fail (pc->f != NULL, 0);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->f != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	return fwrite (buf, sizeof(char), size, pc->f);
 }
@@ -1089,7 +1220,9 @@ gnome_print_bpath (GnomePrintContext * gpc, ArtBpath * bpath, gboolean append)
 
 	g_return_if_fail (gpc != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (gpc));
+	g_return_if_fail (gpc->gc != NULL);
 	g_return_if_fail (bpath != NULL);
+	gnome_print_check_page (gpc);
 
 	if (bpath->code == ART_END) return;
 
@@ -1155,7 +1288,9 @@ gnome_print_vpath (GnomePrintContext * gpc, ArtVpath * vpath, gboolean append)
 
 	g_return_if_fail (gpc != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (gpc));
+	g_return_if_fail (gpc->gc != NULL);
 	g_return_if_fail (vpath != NULL);
+	gnome_print_check_page (gpc);
 
 	if (vpath->code == ART_END) return;
 
@@ -1210,10 +1345,12 @@ gnome_print_vpath (GnomePrintContext * gpc, ArtVpath * vpath, gboolean append)
 int
 gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
 {
-	g_return_val_if_fail (pc != NULL, -1);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
+	g_return_val_if_fail (pc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (pc->gc != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (glyphlist != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_GLYPHLIST (glyphlist), -1);
+	gnome_print_check_page (pc);
 
 	if (((GnomePrintContextClass *) ((GtkObject *) pc)->klass)->glyphlist) {
 		return (* ((GnomePrintContextClass *) ((GtkObject *) pc)->klass)->glyphlist) (pc, glyphlist);
