@@ -1,7 +1,7 @@
 #ifndef __GNOME_PRINTER_DIALOG_H__
 #define __GNOME_PRINTER_DIALOG_H__
 
-#include <libgnomeprint/gnome-printer-profile.h>
+#include <libgnomeprint/gnome-printer.h>
 
 BEGIN_GNOME_DECLS
 
@@ -11,31 +11,15 @@ BEGIN_GNOME_DECLS
 #define GNOME_IS_PRINTER_WIDGET(obj)	     (GTK_CHECK_TYPE ((obj), GNOME_TYPE_PRINTER_WIDGET))
 #define GNOME_IS_PRINTER_WIDGET_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GNOME_TYPE_PRINTER_WIDGET))
 
-typedef struct {
-	GtkFrame frame;
-	
-	GnomePrinterProfileList *profiles;
-	GnomePrinterProfile *profile;
-	GtkWidget *profile_selector;
-	
-	GtkWidget *label_state;
-	GtkWidget *label_type;
-	GtkWidget *label_location;
-	GtkWidget *label_comment;
-	
-	GtkWidget *r1;
-	GtkWidget *r2;
-	GtkWidget *entry_command;
-	GtkWidget *entry_filename;
-	GtkAccelGroup *accel_group;
-} GnomePrinterWidget;
-
-typedef struct {
-	GtkFrameClass parent_class;
-} GnomePrinterWidgetClass;
+typedef struct _GnomePrinterWidget GnomePrinterWidget;
+typedef struct _GnomePrinterWidgetClass GnomePrinterWidgetClass;
 
 GtkType       gnome_printer_widget_get_type    (void);
 GtkWidget    *gnome_printer_widget_new         (void);
+
+void gnome_printer_widget_bind_editable_enters (GnomePrinterWidget * gpw, GnomeDialog * dialog);
+void gnome_printer_widget_bind_accel_group (GnomePrinterWidget * gpw, GtkWindow * window);
+
 GnomePrinter *gnome_printer_widget_get_printer (GnomePrinterWidget *widget);
 
 #define GNOME_TYPE_PRINTER_DIALOG	     (gnome_printer_dialog_get_type ())
@@ -44,15 +28,8 @@ GnomePrinter *gnome_printer_widget_get_printer (GnomePrinterWidget *widget);
 #define GNOME_IS_PRINTER_DIALOG(obj)	     (GTK_CHECK_TYPE ((obj), GNOME_TYPE_PRINTER_DIALOG))
 #define GNOME_IS_PRINTER_DIALOG_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GNOME_TYPE_PRINTER_DIALOG))
 
-typedef struct {
-	GnomeDialog dialog;
-	GnomePrinterWidget *gnome_printer_widget;
-} GnomePrinterDialog;
-
-typedef struct {
-	GnomeDialogClass parent_class;
-} GnomePrinterDialogClass;
-
+typedef struct _GnomePrinterDialog GnomePrinterDialog;
+typedef struct _GnomePrinterDialogClass GnomePrinterDialogClass;
 
 GtkType       gnome_printer_dialog_get_type    (void);
 GnomePrinter *gnome_printer_dialog_new_modal   (void);
