@@ -22,8 +22,8 @@ const char             *gnome_printer_profile_get_driver_name (GnomePrinterProfi
 const char             *gnome_printer_profile_get_output      (GnomePrinterProfile *pp);
 
 GnomePrinter           *gnome_printer_profile_get_printer     (GnomePrinterProfile *pp,
-							       char *optional_file,
-							       char *optional_command);
+							       const char *optional_file,
+							       const char *optional_command);
 
 END_GNOME_DECLS
 
