@@ -1240,7 +1240,7 @@ gdf_find_measured_gdk_font (const gchar * name, GnomeFontWeight weight, gboolean
 		    name, /* Family */
 		    "*", /* Weight */
 		    "*", /* Slant */
-		    "normal", /* Set width */
+              "*", /* Set width */
 		    "*", /* Add Style */
 		    pxsize, /* Pixel Size */
 		    "*", /* Point size */
@@ -1388,8 +1388,10 @@ gdf_measure_x_name (const gchar * xname, GnomeFontWeight weight, gboolean italic
 		g_hash_table_insert (xw2gw, "thin", GINT_TO_POINTER (GNOME_FONT_THIN));
 		g_hash_table_insert (xw2gw, "light", GINT_TO_POINTER (GNOME_FONT_LIGHT));
 		g_hash_table_insert (xw2gw, "regular", GINT_TO_POINTER (GNOME_FONT_BOOK));
+		g_hash_table_insert (xw2gw, "book", GINT_TO_POINTER (GNOME_FONT_BOOK));
 		g_hash_table_insert (xw2gw, "medium", GINT_TO_POINTER (GNOME_FONT_MEDIUM));
 		g_hash_table_insert (xw2gw, "demibold", GINT_TO_POINTER (GNOME_FONT_SEMI));
+		g_hash_table_insert (xw2gw, "semibold", GINT_TO_POINTER (GNOME_FONT_SEMI));
 		g_hash_table_insert (xw2gw, "bold", GINT_TO_POINTER (GNOME_FONT_BOLD));
 		g_hash_table_insert (xw2gw, "heavy", GINT_TO_POINTER (GNOME_FONT_HEAVY));
 		g_hash_table_insert (xw2gw, "extrabold", GINT_TO_POINTER (GNOME_FONT_EXTRABOLD));
