@@ -23,56 +23,61 @@ BEGIN_GNOME_DECLS
 
 typedef struct _GPGC GPGC;
 
+#define GP_GC_FLAG_UNSET 0
+#define GP_GC_FLAG_CHANGED 1
+#define GP_GC_FLAG_CLEAR 2
+
 GPGC * gp_gc_new (void);
 void gp_gc_ref (GPGC * gc);
 void gp_gc_unref (GPGC * gc);
 void gp_gc_reset (GPGC * gc);
 
 /* Stack manipulation */
-
 gint gp_gc_gsave (GPGC * gc);
 gint gp_gc_grestore (GPGC * gc);
 
 /* CTM */
-
 gint gp_gc_setmatrix (GPGC * gc, const gdouble * matrix);
 gint gp_gc_concat (GPGC * gc, const gdouble * matrix);
-
 const gdouble * gp_gc_get_ctm (GPGC * gc);
+gint gp_gc_set_ctm_flag (GPGC * gc, gint flag);
+gint gp_gc_get_ctm_flag (GPGC * gc);
 
 /* Color */
-
 gint gp_gc_set_rgbcolor (GPGC * gc, gdouble r, gdouble g, gdouble b);
 gint gp_gc_set_opacity (GPGC * gc, gdouble opacity);
-
 guint32 gp_gc_get_rgba (GPGC * gc);
 gdouble gp_gc_get_red (GPGC * gc);
 gdouble gp_gc_get_green (GPGC * gc);
 gdouble gp_gc_get_blue (GPGC * gc);
 gdouble gp_gc_get_opacity (GPGC * gc);
+gint gp_gc_set_color_flag (GPGC * gc, gint flag);
+gint gp_gc_get_color_flag (GPGC * gc);
 
 /* Line attributes */
-
 gint gp_gc_set_linewidth (GPGC * gc, gdouble width);
 gint gp_gc_set_miterlimit (GPGC * gc, gdouble limit);
 gint gp_gc_set_linejoin (GPGC * gc, ArtPathStrokeJoinType join);
 gint gp_gc_set_linecap (GPGC * gc, ArtPathStrokeCapType cap);
-gint gp_gc_set_dash (GPGC * gc, int num_values, const gdouble * values, gdouble offset);
-
 gdouble gp_gc_get_linewidth (GPGC * gc);
 gdouble gp_gc_get_miterlimit (GPGC * gc);
 ArtPathStrokeJoinType gp_gc_get_linejoin (GPGC * gc);
 ArtPathStrokeCapType gp_gc_get_linecap (GPGC * gc);
+gint gp_gc_set_line_flag (GPGC * gc, gint flag);
+gint gp_gc_get_line_flag (GPGC * gc);
+gint gp_gc_set_dash (GPGC * gc, int num_values, const gdouble * values, gdouble offset);
 const ArtVpathDash * gp_gc_get_dash (GPGC * gc);
+gint gp_gc_set_dash_flag (GPGC * gc, gint flag);
+gint gp_gc_get_dash_flag (GPGC * gc);
+
 
 /* Font */
-
 gint gp_gc_set_font (GPGC * gc, GnomeFont * font);
-
 const GnomeFont * gp_gc_get_font (GPGC * gc);
+gint gp_gc_set_font_flag (GPGC * gc, gint flag);
+gint gp_gc_get_font_flag (GPGC * gc);
 
 /* Currentpath */
-
 gint gp_gc_newpath (GPGC * gc);
 gint gp_gc_moveto (GPGC * gc, gdouble x, gdouble y);
 gint gp_gc_lineto (GPGC * gc, gdouble x, gdouble y);
