@@ -12,10 +12,14 @@
  */
 
 #include <glib.h>
+#include <libgnome/gnome-defs.h>
+
+BEGIN_GNOME_DECLS
 
 /* 0 if not defined */
 
 gint gp_unicode_from_ps (const gchar * name);
+gint gp_unicode_from_dingbats (const gchar * name);
 
 /* NULL if not defined */
 
@@ -29,5 +33,12 @@ const gchar * gp_const_ps_from_ps (const gchar * name);
 
 const GSList * gp_multi_from_ps (const gchar * name);
 
+/* Get ps name semantics according to Adobe guidelines */
+/* Reference: http://partners.adobe.com/asn/developer/typeforum/unicodegn.html */
+
+GSList * gp_ps_get_semantics (const gchar * name, gboolean * isDecomp, gboolean * isVar);
+
+END_GNOME_DECLS
+
 #endif
 
