@@ -117,7 +117,7 @@ enum parseKey {
   ITALICANGLE, KERNPAIR, KERNPAIRXAMT, LIGATURE, CHARNAME, 
   NOTICE, COMPCHARPIECE, STARTCHARMETRICS, STARTCOMPOSITES, 
   STARTFONTMETRICS, STARTKERNDATA, STARTKERNPAIRS, 
-  STARTTRACKKERN, TRACKKERN, UNDERLINEPOSITION, 
+  STARTTRACKKERN, STDHW, STDVW, TRACKKERN, UNDERLINEPOSITION, 
   UNDERLINETHICKNESS, VERSION, XYWIDTH, XWIDTH, WEIGHT, XHEIGHT,
   NOPE };
 
@@ -142,7 +142,7 @@ static char *keyStrings[] = {
   "ItalicAngle", "KP", "KPX", "L", "N", 
   "Notice", "PCC", "StartCharMetrics", "StartComposites", 
   "StartFontMetrics", "StartKernData", "StartKernPairs", 
-  "StartTrackKern", "TrackKern", "UnderlinePosition", 
+  "StartTrackKern", "StdHW", "StdVW", "TrackKern", "UnderlinePosition", 
   "UnderlineThickness", "Version", "W", "WX", "Weight", "XHeight",
   NULL };
   
@@ -404,6 +404,10 @@ static BOOL parseGlobals(fp, gfi)
                     cont = FALSE;
                     error = normalEOF;
                     break;
+                case STDHW:
+                case STDVW:
+                    keyword = token(fp);
+                    break;
                 case NOPE:
                 default:
                     error = AFM_parseError;
@@ -416,6 +420,7 @@ static BOOL parseGlobals(fp, gfi)
 } /* parseGlobals */    
 
 
+#if 0
 
 /************************* initializeArray ************************/
 
@@ -496,6 +501,7 @@ static int initializeArray(fp, cwi)
     return(error);
         
 } /* initializeArray */    
+#endif
 
 
 /************************* parseCharWidths **************************/
