@@ -1,9 +1,15 @@
+#ifndef _PARSE_AFM_H_
+#define _PARSE_AFM_H_
+
 /* Modified 1999 Morten Welinder:
  * 1. ANSI prototype.
  * 2. parseFileFree function.
  *
  * Modifed 2000 Chema Celorio :
  * 1. add GNOME_DECLS
+ *
+ * Modified 2000 Lauris Kaplinski:
+ * 1. Added #ifdef _PARSE_AFM_H_
  */
 
 /*
@@ -192,11 +198,14 @@ typedef struct
 /* Ligature definition is a linked list since any character can have
  * any number of ligatures.
  */
-typedef struct _t_ligature
+typedef struct _t_ligature Ligature;
+
+struct _t_ligature
 {
-    char *succ, *lig;
-    struct _t_ligature *next;
-} Ligature;
+	char *succ;
+	char *lig;
+	struct _t_ligature *next;
+};
 
 
 /* Character Metric Information. This structure is used only if ALL 
@@ -333,3 +342,5 @@ int parseFile (FILE *fp, Font_Info **fi, FLAGS flags);
 void parseFileFree (Font_Info *fi);
 
 END_GNOME_DECLS
+
+#endif
