@@ -15,7 +15,8 @@ struct _GpaVendor {
 GpaVendor * gpa_vendor_get_from_name (const gchar *vendor_name);
 GpaVendor * gpa_vendor_new (const gchar * name);
 
-gboolean gpa_vendor_list_new_from_file (const gchar *xml_file);
+gboolean  gpa_vendor_list_load_all (void);
+
 gboolean gpa_vendor_verify (GpaVendor *vendor);
 
 END_GNOME_DECLS
