@@ -38,7 +38,7 @@
  **/
 gint
 gpa_tu_replace_all (const gchar *search_text, const gchar *replace_text,
-		gchar **buffer, gboolean case_sensitive)
+		    gchar **buffer, gboolean case_sensitive)
 {
 	gint search_text_length;
 	gint replace_text_length;
@@ -499,7 +499,7 @@ gpa_tu_read_file (const gchar* file_name, gint *size_,
 	
 	f = fopen (file_name, "r");
 	if (f == NULL) {
-		g_warning ("Couldn't open \"%s\"\n", file_name);
+		g_warning ("Couldn't open \"%s\"", file_name);
 		return NULL;
 	}
 	
@@ -794,7 +794,7 @@ gpa_tu_token_previous_dup (const guchar *buffer, gint buffer_length, gint *offse
 
 gchar *
 gpa_tu_token_next_dup_till (const guchar *buffer, gint buffer_size,
-			gint *offset, guchar till_me)
+			    gint *offset, guchar till_me)
 {
 	/* Limit the size of the tokens for buffer overflows */
 	guchar *temp;
@@ -937,3 +937,18 @@ gpa_tu_token_next (const guchar *buffer, gint buffer_size, gint *offset)
 }
 
 
+
+gboolean
+gpa_tu_string_contains_newline (const gchar *token)
+{
+	gint i;
+	gint len;
+
+	len = strlen (token);
+
+	for (i = 0; i < len; i++)
+		if (is_newline (token [i]))
+			return TRUE;
+	
+	return FALSE;	
+}
